/***********************************************************************/
/*  This file is part of the ARM Compiler package                      */
/*  Copyright KEIL ELEKTRONIK GmbH 1992-2003                           */
/***********************************************************************/
/*                                                                     */
/*  flash.ld:  Linker script file for use in Ashling board (Flash).    */
/*                                                                     */
/***********************************************************************/


/*
//*** <<< Use Configuration Wizard in Context Menu >>> *** 
*/


/*
// <h> Memory Configuration
//   <h> Code (Read Only)
//     <o>  Start <0x0-0xFFFFFFFF>
//     <o1> Size  <0x0-0xFFFFFFFF>
//   </h>
//   <h> Data (Read/Write)
//     <o2> Start <0x0-0xFFFFFFFF>
//     <o3> Size  <0x0-0xFFFFFFFF>
//   </h>
// </h>
*/

/* Memory Definitions */

MEMORY
   {
   CODE (rx) : ORIGIN = 0x00000000, LENGTH = 0x00020000
   DATA (rw) : ORIGIN = 0x40000000, LENGTH = 0x00010000
   }


/* Section Definitions */

SECTIONS
   {
   /* First section is .text - used for code */
   .text :
      {
      *startup.o (.text)         /* Startup code */

      *(.text)                   /* Remaining code */

    /*  *(.glue_7t) */                /* Purpose??? */
    /*  *(.glue_7)  */               /* Purpose??? */

      } >CODE =0
      
   . = ALIGN(4); /* Make sure next section starts on a word boundary */

   /* Next, the .rodata section - used for read-only data (constants) */
   .rodata :
     {
     *(.rodata)
     } >CODE

   . = ALIGN(4); /* Make sure next section starts on a word boundary */

   /* Make the current position in memory available for use in startup code */
   /* - Used to reloate the .data section (that is, copy from ROM to RAM) */
   /* - see startup.s */
   _etext = . ; 
   PROVIDE (etext = .); 

   /* Next, the .data section - used for initialised data */
   .data :
     {
     _data = . ;
     *(.data)
     } >DATA

   . = ALIGN(4); /* Make sure next section starts on a word boundary */

   /* Make the current position in memory available for use in startup code */
   /* - Used to reloate the .data section (that is, copy from ROM to RAM) */
   /* - see startup.s */
   _edata = . ;
   PROVIDE (edata = .); 

  /* Next, the .bss section which is used for uninitialised data */
  .bss :
     {
     __bss_start = . ;
     __bss_start__ = . ;
     *(.bss)
     *(COMMON)
     } >DATA

   . = ALIGN(4); /* Make sure next section starts on a word boundary */

  __bss_end__ = . ;
  __bss_end__ = . ;

  _end = .;
  PROVIDE (end = .);
  }  /* End of SECTIONS */

/*------------------------------------------------------------------*-
  ---- END OF FILE -------------------------------------------------
-*------------------------------------------------------------------*/
