/*------------------------------------------------------------------*-

   startup.s (v2.00)

  ------------------------------------------------------------------
  
   Assembly-language startup code for the LPC2100.

   This file is adapted from an example in the uVision/ARM 
   development tools (copyright KEIL ELEKTRONIK GmbH 2002-2003)
   
   It contains additional material by:
      Musharraf Ahmed Hanif
      Sameer Pal
      Michael J. Pont
      He Wan

-*------------------------------------------------------------------*/

/* Specify required stack sizes (for each mode) 
   NOTE: You need to check the stack sizes - see below */

   /* Address of the top of the stack area */
   .equ    Top_Stack, 0x40004000

   /* Sizes of the different stack regions (in bytes) 
      NOTE: Use hex if you prefer! */
   .equ    UND_Stk, 4 
   .equ    SVC_Stk, 4 
   .equ    ABT_Stk, 4 
   .equ    FIQ_Stk, 256 
   .equ    IRQ_Stk, 4 
   .equ    USR_Stk, 2048 

/* Startup Code must be linked first at the address at which 
   it expects to run. */
   .text
   .arm

   .global _startup
   .func   _startup
_startup:

/* This is the "vector table" which defines the Program Counter (PC) 
   values that will be used to invoke the appropriate ISRs in the event 
   of an interrupt.  At startup, after the program reaches this point,
   the PC will be set to point to the reset address (see below), and 
   program execution will "jump over" the rest of the vector table 

   (Mapped to address 0.  Absolute addressing mode must be used.)
   Unused interrupts are linked to infinite loops.  */

Vectors:        LDR     PC, Reset_Addr         
                LDR     PC, Undef_Addr
                LDR     PC, SWI_Addr
                LDR     PC, PAbt_Addr
                LDR     PC, DAbt_Addr
                NOP                         /* Reserved Vector */
                LDR     PC, IRQ_Addr
                LDR     PC, FIQ_Addr

Reset_Addr:     .word   Reset_Handler
Undef_Addr:     .word   Undef_Handler
SWI_Addr:       .word   SWI_Handler
PAbt_Addr:      .word   PAbt_Handler
DAbt_Addr:      .word   DAbt_Handler
                .word   0                   /* Reserved Address */
IRQ_Addr:       .word   IRQ_Handler
FIQ_Addr:       .word   FIQ_Handler

Undef_Handler:  B       Undef_Handler
SWI_Handler:    B       SWI_Handler
PAbt_Handler:   B       PAbt_Handler
DAbt_Handler:   B       DAbt_Handler
IRQ_Handler:    B       IRQ_Handler
FIQ_Handler:    B       FIQ_Handler            /* See FIQ_Wrapper.s */

/* Reset Handler */
Reset_Handler:  

/* End of vector table */

/* Now we fill the stack areas with known values (to allow stack checking) 
   First calculate the total stack depth (sum of all stacks);
   store result in R0 */
   LDR R0, =Top_Stack -UND_Stk -SVC_Stk -ABT_Stk -FIQ_Stk -IRQ_Stk -USR_Stk

   /* Stack will be filled with the "hex string" DEADC0DE (by tradition!) */
   /* NOTE: "little endian" byte ordering is assumed here. */
                LDR R1, = 0xDEC0ADDE   

				LDR		R2, =Top_Stack
Stack_filler:	STR		R1, [R0]
				ADD		R0, R0, #4
				CMP		R0, R2
				BNE		Stack_filler   

/* End of stack-filler */


/* Now we need to initialise the interrupt system */

        /* Mode bits in program status registers */
        .equ    Mode_USR,       0x10
        .equ    Mode_FIQ,       0x11
        .equ    Mode_IRQ,       0x12
        .equ    Mode_SVC,       0x13
        .equ    Mode_ABT,       0x17
        .equ    Mode_UND,       0x1B
        .equ    Mode_SYS,       0x1F

        /* Interrupt flags in program status registers */
        .equ    I_Bit,          0x80 /* when I bit set, IRQ is disabled */
        .equ    F_Bit,          0x40 /* when F bit set, FIQ is disabled */

        /* Now set up the stack for each mode */
                LDR     R0, =Top_Stack

                /* Enter Undefined Instruction Mode and set its stack ptr */
                MSR     CPSR_c, #Mode_UND|I_Bit|F_Bit
                MOV     SP, R0
                SUB     R0, R0, #UND_Stk

                /* Enter Abort Mode and set its stack ptr */
                MSR     CPSR_c, #Mode_ABT|I_Bit|F_Bit
                MOV     SP, R0
                SUB     R0, R0, #ABT_Stk

                /* Enter FIQ Mode and set its stack ptr */
                MSR     CPSR_c, #Mode_FIQ|I_Bit|F_Bit
                MOV     SP, R0
                SUB     R0, R0, #FIQ_Stk

                /* Enter IRQ Mode and set its Stack ptr */
                MSR     CPSR_c, #Mode_IRQ|I_Bit|F_Bit
                MOV     SP, R0
                SUB     R0, R0, #IRQ_Stk

                /* Enter Supervisor Mode and set its stack ptr */
                MSR     CPSR_c, #Mode_SVC|I_Bit|F_Bit
                MOV     SP, R0
                SUB     R0, R0, #SVC_Stk

                /* Enter User Mode and set its stack ptr */
                MSR     CPSR_c, #Mode_USR
                MOV     SP, R0

/* Relocate the .data section (Copy from ROM to RAM) */
                LDR     R1, =_etext
                LDR     R2, =_data
                LDR     R3, =_edata
LoopRel:        CMP     R2, R3
                LDRLO   R0, [R1], #4
                STRLO   R0, [R2], #4
                BLO     LoopRel


/* Clear .bss section (Zero init) */
                MOV     R0, #0
                LDR     R1, =__bss_start__
                LDR     R2, =__bss_end__
LoopZI:         CMP     R1, R2
                STRLO   R0, [R1], #4
                BLO     LoopZI


/* Finally, start running the C code - branch to main() */
                B       main

        .size   _startup, . - _startup
        .endfunc

        .end

/*------------------------------------------------------------------*-
  ---- END OF FILE -------------------------------------------------
-*------------------------------------------------------------------*/
